@@ -1,7 +1,9 @@
 package Progress::Any;
 
-our $DATE = '2018-03-26'; # DATE
-our $VERSION = '0.214'; # VERSION
+our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
+our $DATE = '2020-08-15'; # DATE
+our $DIST = 'Progress-Any'; # DIST
+our $VERSION = '0.219'; # VERSION
 
 use 5.010001;
 use strict;
@@ -43,7 +45,7 @@ our %outputs;     # key = task name, value = [$outputobj, ...]
 
 # return 1 if created, 0 if already created/initialized
 sub _init_indicator {
-    my ($class, $task) = @_;
+    my ($class, $task, $default_target) = @_;
 
     #say "D: _init_indicator($task)";
 
@@ -53,7 +55,7 @@ sub _init_indicator {
     my $progress = bless({
         task        => $task,
         title       => $task,
-        target      => undef,
+        target      => $default_target,
         pos         => 0,
         state       => 'stopped',
 
@@ -66,7 +68,7 @@ sub _init_indicator {
 
     # if we create an indicator named a.b.c, we must also create a.b, a, and ''.
     if ($task =~ s/\.?\w+\z//) {
-        $class->_init_indicator($task);
+        $class->_init_indicator($task, 0);
     }
 
     $progress;
@@ -359,7 +361,10 @@ sub _should_update_output {
         # finishing, update the output to show finished state
         return 1;
     } elsif ($output->{force_update}) {
-        # force update
+        # this is an undocumented force update for now, the output itself or
+        # something else can set this to force an update. but this will only be
+        # done once because we delete the key; if another update wants to be
+        # forced, they need to set this again.
         delete $output->{force_update};
         return 1;
     } elsif ($priority eq 'high') {
@@ -393,6 +398,15 @@ sub update {
 
     my $message  = delete($args{message});
     my $priority = delete($args{priority}) // 'normal';
+    my $force_update = delete($args{force_update});
+
+    my %other_args_for_output;
+    for (keys %args) {
+        if (/\Amessage\.alt\./) {
+            $other_args_for_output{$_} = delete $args{$_};
+        }
+    }
+
     die "Unknown argument(s) to update(): ".join(", ", keys(%args))
         if keys(%args);
 
@@ -405,7 +419,8 @@ sub update {
         while (1) {
             if ($outputs{$task}) {
                 for my $output (@{ $outputs{$task} }) {
-                    next unless $self->_should_update_output($output, $now, $priority);
+                    next unless $force_update ||
+                        $self->_should_update_output($output, $now, $priority);
                     if (ref($message) eq 'CODE') {
                         $message = $message->();
                     }
@@ -414,6 +429,12 @@ sub update {
                         message   => $message,
                         priority  => $priority,
                         time      => $now,
+
+                        # temporary, internal API. to let an output module know
+                        # the same update() when there are multiple instances of
+                        # it
+                        _update_id => $now,
+                        %other_args_for_output,
                     );
                     $output->{_mtime} = $now;
                     $output->{_pos}   = $pos;
@@ -439,6 +460,11 @@ sub finish {
     $self->update(pos=>$self->{target}, state=>'finished', %args);
 }
 
+sub reset {
+    my ($self, %args) = @_;
+    $self->update(pos=>0, state=>'started', %args);
+}
+
 our $template_regex = qr{( # all=1
                              %
                              ( #width=2
@@ -585,7 +611,7 @@ Progress::Any - Record progress to any output
 
 =head1 VERSION
 
-This document describes version 0.214 of Progress::Any (from Perl distribution Progress-Any), released on 2018-03-26.
+This document describes version 0.219 of Progress::Any (from Perl distribution Progress-Any), released on 2020-08-15.
 
 =head1 SYNOPSIS
 
@@ -607,7 +633,7 @@ Sample output:
  % ./script.pl
   60% [Doing item 6====           ]3s left
 
-Another example, this time with terminal message as output (see :
+Another example, this time with terminal message as output:
 
  use Progress::Any '$progress';
  use Progress::Any::Output 'TermMessage', template => '[%n] %P/%T (%6.2p%%) %m';
@@ -904,6 +930,11 @@ ignore updates lower than a certain level.
 
 Can be set to C<finished> to finish a task.
 
+=item * force_update => BOOL
+
+Default false. Some outputs choose only to update themselves after a certain
+amount of time or number of updates have passed; this forces their update.
+
 =back
 
 =head2 finish
@@ -920,6 +951,20 @@ Equivalent to:
      %args,
  );
 
+=head2 reset
+
+Usage:
+
+ $progress->reset(%args)
+
+Equivalent to:
+
+ $progress->update(
+     pos => 0,
+     state => 'started',
+     %args,
+ );
+
 =head2 start
 
 Usage:
@@ -1093,6 +1138,8 @@ feature.
 
 =head1 SEE ALSO
 
+L<Progress::Any::Examples> distribution contains example scripts.
+
 Other progress modules on CPAN: L<Term::ProgressBar>,
 L<Term::ProgressBar::Simple>, L<Time::Progress>, among others.
 
@@ -1107,7 +1154,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018, 2015, 2014, 2013, 2012 by perlancar@cpan.org.
+This software is copyright (c) 2020, 2018, 2015, 2014, 2013, 2012 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
